#ifndef mail_message_h
#define mail_message_h

#include <string>
#include <vector>

class MailMessage {
public:

      // Constructor reads the headers from stdin, stopping at a
      // empty line.
      //
   MailMessage();

      // Destructor cleans things up, reading and discarding the message
      // body if it has not yet been read (to prevent upstream broken
      // pipe errors).
      //
   ~MailMessage();

      // Set the state of the X-TrollFilter header (true means Yes,
      // false means No).
      //
   void
   SetTrollFilter(bool init_is_troll_message);

      // Write the headers we read during constructor to stdout, adding the
      // X-TrollFilter at the end, then write a blank line and
      // copy the message body from stdin to stdout.
      //
   void
   WriteToStdout();

      // Return the From: email address from the message headers.
      //
   std::string
   FromAddress();

      // Return the Message-ID: string from the message headers.
      //
   std::string
   MessageID();

      // Return array of References: message IDs from the message headers.
      //
   std::vector<std::string> &
   References();

      // Return array of Received: headers from the message headers.
      //
   std::vector<std::string> &
   Received();

      // Return X-Bogosity header string (or null)
      //
   std::string
   Bogosity();

      // Set the X-External-Spam header string.
      //
   void
   SetSpamIP(std::string ipaddr);

private:

      // Read headers from stdin, leave body sitting on stdin till ready to
      // pass it on to stdout or discard it.
      //
   void
   InitFromStdin();

      // Read and discard stdin till EOF.
      //
   void
   DiscardStdin();

      // Look through the header_lines starting at line "start_line" for
      // a header with the name "which_header". If found, place the
      // contents of that header in "hdr" (handling any continuation
      // lines that follow it), return the index of the next line
      // to look at following that header. Return -1 if the header
      // was not found.
      //
   int
   SuckHeader(const char * which_header, std::string & hdr, int start_line = 0);

   bool message_copied;

   bool have_received;
   std::vector<std::string> received;

   bool have_bogosity;
   std::string bogosity;

   bool have_message_id;
   std::string message_id;

   bool have_references;
   std::vector<std::string> references;

   bool have_from_address;
   std::string from_address;

   std::string spam_ip;

   std::vector<std::string> header_lines;
   const char * endstr;

   bool is_troll_message;
};

#endif // mail_message_h
